@@ -187,9 +187,9 @@ OFFSET_TABLE_DTYPE = [ ("ID",       np.uint32),
                        ("EndRow",   np.uint32) ]
 
 
-def _makePulseFeatureAccessor(featureName):
+def _makeBaseFeatureAccessor(featureName):
     def f(self, aligned=True, orientation="native"):
-        return self.pulseFeature(featureName, aligned, orientation)
+        return self.baseFeature(featureName, aligned, orientation)
     return f
 
 class CmpH5Alignment(AlignmentRecordMixin):
@@ -546,9 +546,9 @@ class CmpH5Alignment(AlignmentRecordMixin):
         else:
             return self.rStart + np.hstack([0, np.cumsum(readNonGapMask[:-1])])
 
-    def pulseFeature(self, featureName, aligned=True, orientation="native"):
+    def baseFeature(self, featureName, aligned=True, orientation="native"):
         """
-        Access a pulse feature by name.
+        Access a base feature by name.
         """
         pulseDataset = self._alignmentGroup[featureName]
         pulseArray = arrayFromDataset(pulseDataset, self.Offset_begin, self.Offset_end)
@@ -561,14 +561,14 @@ class CmpH5Alignment(AlignmentRecordMixin):
         else:
             return ungappedPulseArray(alignedPulseArray)
 
-    IPD            = _makePulseFeatureAccessor("IPD")
-    PulseWidth     = _makePulseFeatureAccessor("PulseWidth")
-    QualityValue   = _makePulseFeatureAccessor("QualityValue")
-    InsertionQV    = _makePulseFeatureAccessor("InsertionQV")
-    DeletionQV     = _makePulseFeatureAccessor("DeletionQV")
-    DeletionTag    = _makePulseFeatureAccessor("DeletionTag")
-    MergeQV        = _makePulseFeatureAccessor("MergeQV")
-    SubstitutionQV = _makePulseFeatureAccessor("SubstitutionQV")
+    IPD            = _makeBaseFeatureAccessor("IPD")
+    PulseWidth     = _makeBaseFeatureAccessor("PulseWidth")
+    QualityValue   = _makeBaseFeatureAccessor("QualityValue")
+    InsertionQV    = _makeBaseFeatureAccessor("InsertionQV")
+    DeletionQV     = _makeBaseFeatureAccessor("DeletionQV")
+    DeletionTag    = _makeBaseFeatureAccessor("DeletionTag")
+    MergeQV        = _makeBaseFeatureAccessor("MergeQV")
+    SubstitutionQV = _makeBaseFeatureAccessor("SubstitutionQV")
 
     def __getattr__(self, key):
         return self.cmpH5.alignmentIndex[self.rowNumber][key]
@@ -755,8 +755,8 @@ class CmpH5Reader(ReaderBase, IndexedAlignmentReaderMixin):
         # dataset_name -> dataset }.  This way we avoid B-tree
         # scanning in basic data access.
         self._alignmentGroupById = {}
-        for (alnGroupId, alnGroupPath) in zip(self.file["/AlnGroup/ID"],
-                                              self.file["/AlnGroup/Path"]):
+        for (alnGroupId, alnGroupPath) in zip(self.file["/AlnGroup/ID"][:],
+                                              self.file["/AlnGroup/Path"][:]):
             alnGroup = self.file[alnGroupPath]
             self._alignmentGroupById[alnGroupId] = dict(alnGroup.items())
 
@@ -807,18 +807,18 @@ class CmpH5Reader(ReaderBase, IndexedAlignmentReaderMixin):
 
     def _loadReferenceInfo(self):
         _referenceGroupTbl = np.rec.fromrecords(
-            zip(self.file["/RefGroup/ID"],
-                self.file["/RefGroup/RefInfoID"],
+            zip(self.file["/RefGroup/ID"][:],
+                self.file["/RefGroup/RefInfoID"][:],
                 [path[1:] for path in self.file["/RefGroup/Path"]]),
             dtype=[("ID"       , int),
                    ("RefInfoID", int),
                    ("Name"     , object)])
 
         _referenceInfoTbl = np.rec.fromrecords(
-            zip(self.file["/RefInfo/ID"],
-                self.file["/RefInfo/FullName"],
-                self.file["/RefInfo/Length"],
-                self.file["/RefInfo/MD5"]) ,
+            zip(self.file["/RefInfo/ID"][:],
+                self.file["/RefInfo/FullName"][:],
+                self.file["/RefInfo/Length"][:],
+                self.file["/RefInfo/MD5"][:]) ,
             dtype=[("RefInfoID", int),
                    ("FullName" , object),
                    ("Length"   , int),
@@ -838,26 +838,37 @@ class CmpH5Reader(ReaderBase, IndexedAlignmentReaderMixin):
                                                 jointype="inner")
         self._referenceDict = {}
         self._readLocatorByKey = {}
-        for record in self._referenceInfoTable:
-            if record.ID != -1:
-                assert record.ID != record.Name
+
+        # For cmp.h5 files with large numbers of references, accessing
+        # the recarray fields in the inner loop was terribly slow.
+        # This makes things faster, though the code is less
+        # straightforward.  (One of the tradeoffs we have to make
+        # without a compiler to help us...)
+        recordID       = self._referenceInfoTable.ID
+        recordName     = self._referenceInfoTable.Name
+        recordFullName = self._referenceInfoTable.FullName
+        recordMD5      = self._referenceInfoTable.MD5
+
+        for i, record in enumerate(self._referenceInfoTable):
+            if recordID[i] != -1:
+                assert recordID[i] != record.Name
                 shortName = splitFastaHeader(record.FullName)[0]
-                if (shortName       in self._referenceDict or
-                    record.ID       in self._referenceDict or
-                    record.Name     in self._referenceDict or
-                    record.FullName in self._referenceDict or
-                    record.MD5      in self._referenceDict):
+                if (shortName         in self._referenceDict or
+                    recordID[i]       in self._referenceDict or
+                    recordName[i]     in self._referenceDict or
+                    recordFullName[i] in self._referenceDict or
+                    recordMD5[i]      in self._referenceDict):
                     raise ValueError, "Duplicate reference contig sequence or identifier"
                 else:
-                    self._referenceDict[shortName]       = record
-                    self._referenceDict[record.ID]       = record
-                    self._referenceDict[record.Name]     = record
-                    self._referenceDict[record.FullName] = record
-                    self._referenceDict[record.MD5]      = record
+                    self._referenceDict[shortName]         = record
+                    self._referenceDict[recordID[i]]       = record
+                    self._referenceDict[recordName[i]]     = record
+                    self._referenceDict[recordFullName[i]] = record
+                    self._referenceDict[recordMD5[i]]      = record
 
                 if self.isSorted:
-                    readLocator = makeReadLocator(self, record.ID)
-                    self._readLocatorByKey[record.ID] = readLocator
+                    readLocator = makeReadLocator(self, recordID[i])
+                    self._readLocatorByKey[recordID[i]] = readLocator
                     self._readLocatorByKey[shortName] = readLocator
 
     def _loadMiscInfo(self):
@@ -1169,7 +1180,7 @@ class CmpH5Reader(ReaderBase, IndexedAlignmentReaderMixin):
         else:
             return self[rowNumbers]
 
-    def hasPulseFeature(self, featureName):
+    def hasBaseFeature(self, featureName):
         """
         Are the datasets for pulse feature `featureName` loaded in
         this file?  Specifically, is it loaded for all movies within
@@ -1177,30 +1188,30 @@ class CmpH5Reader(ReaderBase, IndexedAlignmentReaderMixin):
 
         .. doctest::
 
-            >>> c.hasPulseFeature("InsertionQV")
+            >>> c.hasBaseFeature("InsertionQV")
             True
-            >>> c.hasPulseFeature("MergeQV")
+            >>> c.hasBaseFeature("MergeQV")
             False
 
         """
         return all(featureName in alnGroup.keys()
                    for alnGroup in self._alignmentGroupById.values())
 
-    def pulseFeaturesAvailable(self):
+    def baseFeaturesAvailable(self):
         """
         What pulse features are available in this cmp.h5 file?
 
         .. doctest::
 
-            >>> c.pulseFeaturesAvailable()
+            >>> c.baseFeaturesAvailable()
             [u'QualityValue', u'IPD', u'PulseWidth', u'InsertionQV', u'DeletionQV']
 
         """
-        pulseFeaturesByMovie = [ alnGroup.keys()
+        baseFeaturesByMovie = [ alnGroup.keys()
                                  for alnGroup in self._alignmentGroupById.values() ]
-        pulseFeaturesAvailableAsSet = set.intersection(*map(set, pulseFeaturesByMovie))
-        pulseFeaturesAvailableAsSet.discard("AlnArray")
-        return list(pulseFeaturesAvailableAsSet)
+        baseFeaturesAvailableAsSet = set.intersection(*map(set, baseFeaturesByMovie))
+        baseFeaturesAvailableAsSet.discard("AlnArray")
+        return list(baseFeaturesAvailableAsSet)
 
     @property
     def barcode(self):
