@@ -5,15 +5,18 @@ import unittest
 import tempfile
 import os
 import itertools
+import numpy as np
+
+import pysam
 
 from pbcore.util.Process import backticks
 from pbcore.io.dataset.utils import (consolidateBams, _infixFname,
                                     BamtoolsVersion)
 from pbcore.io import (DataSet, SubreadSet, ConsensusReadSet,
-                       ReferenceSet, ContigSet, AlignmentSet,
+                       ReferenceSet, ContigSet, AlignmentSet, BarcodeSet,
                        FastaReader, FastaWriter, IndexedFastaReader,
                        HdfSubreadSet, ConsensusAlignmentSet,
-                       openDataFile, FastaWriter)
+                       openDataFile, FastaWriter, FastqReader)
 import pbcore.data as upstreamData
 import pbcore.data.datasets as data
 from pbcore.io.dataset.DataSetValidator import validateXml
@@ -22,11 +25,6 @@ import xml.etree.ElementTree as ET
 log = logging.getLogger(__name__)
 
 def _check_constools():
-    cmd = "dataset.py"
-    o, r, m = backticks(cmd)
-    if r != 2:
-        return False
-
     if not BamtoolsVersion().good:
         log.warn("Bamtools not found or out of date")
         return False
@@ -53,8 +51,8 @@ class TestDataSet(unittest.TestCase):
 
 
     def test_subread_build(self):
-        ds1 = SubreadSet(data.getXml(no=5))
-        ds2 = SubreadSet(data.getXml(no=5))
+        ds1 = SubreadSet(data.getXml(no=5), skipMissing=True)
+        ds2 = SubreadSet(data.getXml(no=5), skipMissing=True)
         self.assertEquals(type(ds1).__name__, 'SubreadSet')
         self.assertEquals(ds1._metadata.__class__.__name__,
                           'SubreadSetMetadata')
@@ -64,7 +62,7 @@ class TestDataSet(unittest.TestCase):
         self.assertEquals(len(ds2.metadata.collections), 1)
         ds3 = ds1 + ds2
         self.assertEquals(len(ds3.metadata.collections), 2)
-        ds4 = SubreadSet(data.getSubreadSet())
+        ds4 = SubreadSet(data.getSubreadSet(), skipMissing=True)
         self.assertEquals(type(ds4).__name__, 'SubreadSet')
         self.assertEquals(type(ds4._metadata).__name__, 'SubreadSetMetadata')
         self.assertEquals(len(ds4.metadata.collections), 1)
@@ -136,11 +134,33 @@ class TestDataSet(unittest.TestCase):
         for name in names:
             self.assertTrue(ds[name].id == name)
 
+    def test_contigset_split(self):
+        ref = ReferenceSet(data.getXml(9))
+        exp_n_contigs = len(ref)
+        refs = ref.split(10)
+        self.assertEqual(len(refs), 10)
+        obs_n_contigs = 0
+        for r in refs:
+            obs_n_contigs += sum(1 for _ in r)
+        self.assertEqual(obs_n_contigs, exp_n_contigs)
+
+
+    def test_contigset_len(self):
+        ref = ReferenceSet(data.getXml(9))
+        exp_n_contigs = len(ref)
+        refs = ref.split(10)
+        self.assertEqual(len(refs), 10)
+        obs_n_contigs = 0
+        for r in refs:
+            obs_n_contigs += len(r)
+        self.assertEqual(obs_n_contigs, exp_n_contigs)
+
+
     def test_ccsread_build(self):
-        ds1 = ConsensusReadSet(data.getXml(2), strict=False)
+        ds1 = ConsensusReadSet(data.getXml(2), strict=False, skipMissing=True)
         self.assertEquals(type(ds1).__name__, 'ConsensusReadSet')
         self.assertEquals(type(ds1._metadata).__name__, 'SubreadSetMetadata')
-        ds2 = ConsensusReadSet(data.getXml(2), strict=False)
+        ds2 = ConsensusReadSet(data.getXml(2), strict=False, skipMissing=True)
         self.assertEquals(type(ds2).__name__, 'ConsensusReadSet')
         self.assertEquals(type(ds2._metadata).__name__, 'SubreadSetMetadata')
 
@@ -161,7 +181,8 @@ class TestDataSet(unittest.TestCase):
         ds1.write(fn)
 
     def test_ccsalignment_build(self):
-        ds1 = ConsensusAlignmentSet(data.getXml(20), strict=False)
+        ds1 = ConsensusAlignmentSet(data.getXml(20), strict=False,
+                                    skipMissing=True)
         self.assertEquals(type(ds1).__name__, 'ConsensusAlignmentSet')
         self.assertEquals(type(ds1._metadata).__name__, 'SubreadSetMetadata')
         # XXX strict=True requires actual existing .bam files
@@ -223,10 +244,10 @@ class TestDataSet(unittest.TestCase):
 
 
     def test_contigset_build(self):
-        ds1 = ContigSet(data.getXml(3))
+        ds1 = ContigSet(data.getXml(3), skipMissing=True)
         self.assertEquals(type(ds1).__name__, 'ContigSet')
         self.assertEquals(type(ds1._metadata).__name__, 'ContigSetMetadata')
-        ds2 = ContigSet(data.getXml(3))
+        ds2 = ContigSet(data.getXml(3), skipMissing=True)
         self.assertEquals(type(ds2).__name__, 'ContigSet')
         self.assertEquals(type(ds2._metadata).__name__, 'ContigSetMetadata')
         for contigmd in ds2.metadata.contigs:
@@ -240,7 +261,22 @@ class TestDataSet(unittest.TestCase):
         self.assertEqual(len(aln.toExternalFiles()), 2)
         outdir = tempfile.mkdtemp(suffix="dataset-unittest")
         outfn = os.path.join(outdir, 'merged.bam')
-        consolidateBams(aln.toExternalFiles(), outfn, filterDset=aln)
+        consolidateBams(aln.toExternalFiles(), outfn, filterDset=aln,
+                        useTmp=False)
+        self.assertTrue(os.path.exists(outfn))
+        consAln = AlignmentSet(outfn)
+        self.assertEqual(len(consAln.toExternalFiles()), 1)
+        for read1, read2 in zip(sorted(list(aln)), sorted(list(consAln))):
+            self.assertEqual(read1, read2)
+        self.assertEqual(len(aln), len(consAln))
+
+        log.debug("Test methods directly in tmp")
+        aln = AlignmentSet(data.getXml(12))
+        self.assertEqual(len(aln.toExternalFiles()), 2)
+        outdir = tempfile.mkdtemp(suffix="dataset-unittest")
+        outfn = os.path.join(outdir, 'merged.bam')
+        consolidateBams(aln.toExternalFiles(), outfn, filterDset=aln,
+                        useTmp=True)
         self.assertTrue(os.path.exists(outfn))
         consAln = AlignmentSet(outfn)
         self.assertEqual(len(consAln.toExternalFiles()), 1)
@@ -326,16 +362,17 @@ class TestDataSet(unittest.TestCase):
             self.assertEqual(read1, read2)
         self.assertEqual(len(list(aln)), len(list(nonCons)))
 
-        log.debug("Test cli")
-        outdir = tempfile.mkdtemp(suffix="dataset-unittest")
-        datafile = os.path.join(outdir, "merged.bam")
-        xmlfile = os.path.join(outdir, "merged.xml")
-        cmd = "dataset.py consolidate {i} {d} {x}".format(i=data.getXml(12),
-                                                          d=datafile,
-                                                          x=xmlfile)
-        log.debug(cmd)
-        o, r, m = backticks(cmd)
-        self.assertEqual(r, 0)
+    def test_accuracy_filter(self):
+        aln = AlignmentSet(data.getXml(12))
+        self.assertEqual(len(list(aln)), 177)
+        aln.filters.addRequirement(accuracy=[('>', '.85')])
+        self.assertEqual(len(list(aln)), 174)
+
+    def test_contigset_filter(self):
+        ref = ReferenceSet(data.getXml(9))
+        self.assertEqual(len(list(ref)), 59)
+        ref.filters.addRequirement(length=[('>', '1450')])
+        self.assertEqual(len(list(ref)), 34)
 
     @unittest.skipIf(not _check_constools() or not _internal_data(),
                      "bamtools, pbindex or data not found, skipping")
@@ -360,15 +397,6 @@ class TestDataSet(unittest.TestCase):
             self.assertEqual(read1, read2)
         self.assertEqual(len(aln), len(nonCons))
 
-        log.debug("Test cli")
-        outdir = tempfile.mkdtemp(suffix="dataset-unittest")
-        datafile = os.path.join(outdir, "merged.bam")
-        xmlfile = os.path.join(outdir, "merged.xml")
-        cmd = "dataset.py consolidate --numFiles 2 {i} {d} {x}".format(
-            i=testFile, d=datafile, x=xmlfile)
-        log.debug(cmd)
-        o, r, m = backticks(cmd)
-        self.assertEqual(r, 0)
 
     @unittest.skipIf(not _check_constools(),
                      "bamtools or pbindex not found, skipping")
@@ -435,7 +463,10 @@ class TestDataSet(unittest.TestCase):
         exp_single_seqs = [rec.sequence for rec in exp_singles]
 
         acc_file = ContigSet(outFas1, outFas2)
+        acc_file.induceIndices()
         log.debug(acc_file.toExternalFiles())
+        self.assertEqual(len(acc_file), 4)
+        self.assertEqual(len(list(acc_file)), 4)
         acc_file.consolidate()
         log.debug(acc_file.toExternalFiles())
 
@@ -445,6 +476,47 @@ class TestDataSet(unittest.TestCase):
         self.assertEqual(acc_file.get_contig(double).sequence[:],
                          exp_double_seq)
 
+        self.assertEqual(len(acc_file._openReaders), 1)
+        self.assertEqual(len(acc_file.index), 3)
+        self.assertEqual(len(acc_file._indexMap), 3)
+        self.assertEqual(len(acc_file), 3)
+        self.assertEqual(len(list(acc_file)), 3)
+
+    def test_contigset_consolidate_int_names(self):
+        #build set to merge
+        outdir = tempfile.mkdtemp(suffix="dataset-unittest")
+
+        inFas = os.path.join(outdir, 'infile.fasta')
+        outFas1 = os.path.join(outdir, 'tempfile1.fasta')
+        outFas2 = os.path.join(outdir, 'tempfile2.fasta')
+
+        # copy fasta reference to hide fai and ensure FastaReader is used
+        backticks('cp {i} {o}'.format(
+                      i=ReferenceSet(data.getXml(9)).toExternalFiles()[0],
+                      o=inFas))
+        rs1 = ContigSet(inFas)
+
+        double = 'B.cereus.1'
+        exp_double = rs1.get_contig(double)
+
+        # todo: modify the names first:
+        with FastaWriter(outFas1) as writer:
+            writer.writeRecord('5141', exp_double.sequence)
+        with FastaWriter(outFas2) as writer:
+            writer.writeRecord('5142', exp_double.sequence)
+
+        exp_double_seqs = [exp_double.sequence, exp_double.sequence]
+        exp_names = ['5141', '5142']
+
+        obs_file = ContigSet(outFas1, outFas2)
+        log.debug(obs_file.toExternalFiles())
+        obs_file.consolidate()
+        log.debug(obs_file.toExternalFiles())
+
+        # open obs and compare to exp
+        for name, seq in zip(exp_names, exp_double_seqs):
+            self.assertEqual(obs_file.get_contig(name).sequence[:], seq)
+
     def test_contigset_consolidate_genomic_consensus(self):
         """
         Verify that the contigs output by GenomicConsensus (e.g. quiver) can
@@ -459,20 +531,20 @@ class TestDataSet(unittest.TestCase):
         files = []
         for i, (header, seq) in enumerate([FASTA1, FASTA2, FASTA3]):
             _files = []
-            for suffix in ["", "|quiver", "|plurality", "|arrow"]:
+            for suffix in ["", "|quiver", "|plurality", "|arrow", "|poa"]:
                 tmpfile = tempfile.NamedTemporaryFile(suffix=".fasta").name
                 with open(tmpfile, "w") as f:
                     f.write(">{h}{s}\n{q}".format(h=header, s=suffix, q=seq))
                 _files.append(tmpfile)
             files.append(_files)
         for i in range(3):
-            ds = ContigSet(*[ f[i] for f in files ])
+            ds = ContigSet(*[f[i] for f in files])
             out1 = tempfile.NamedTemporaryFile(suffix=".contigset.xml").name
             fa1 = tempfile.NamedTemporaryFile(suffix=".fasta").name
             ds.consolidate(fa1)
             ds.write(out1)
             with ContigSet(out1) as ds_new:
-                self.assertEqual(len([ rec for rec in ds_new ]), 1,
+                self.assertEqual(len([rec for rec in ds_new]), 1,
                                  "failed on %d" % i)
 
     def test_split_hdfsubreadset(self):
@@ -483,6 +555,52 @@ class TestDataSet(unittest.TestCase):
         self.assertEqual(len(hdfdss[0].toExternalFiles()), 2)
         self.assertEqual(len(hdfdss[1].toExternalFiles()), 1)
 
+    @unittest.skipIf(not _internal_data(),
+                     "Internal data not found, skipping")
+    def test_len_fastq(self):
+        fn = ('/pbi/dept/secondary/siv/testdata/SA3-RS/'
+              'lambda/2590980/0008/Analysis_Results/'
+              'm141115_075238_ethan_c100699872550000001'
+              '823139203261572_s1_p0.1.subreads.fastq')
+        fq_out = tempfile.NamedTemporaryFile(suffix=".fastq").name
+        with open(fq_out, 'w') as fqh:
+            with open(fn, 'r') as fih:
+                for line in itertools.islice(fih, 24):
+                    fqh.write(line)
+        cset = ContigSet(fq_out)
+        self.assertFalse(cset.isIndexed)
+        self.assertTrue(isinstance(cset.resourceReaders()[0], FastqReader))
+        self.assertEqual(sum(1 for _ in cset),
+                         sum(1 for _ in FastqReader(fq_out)))
+        self.assertEqual(sum(1 for _ in cset), 6)
+        # XXX not possible, fastq files can't be indexed:
+        #self.assertEqual(len(cset), sum(1 for _ in cset))
+
+    @unittest.skipIf(not _internal_data(),
+                     "Internal data not found, skipping")
+    def test_fastq_consolidate(self):
+        fn = ('/pbi/dept/secondary/siv/testdata/SA3-RS/'
+              'lambda/2590980/0008/Analysis_Results/'
+              'm141115_075238_ethan_c100699872550000001'
+              '823139203261572_s1_p0.1.subreads.fastq')
+        fq_out = tempfile.NamedTemporaryFile(suffix=".fastq").name
+        cfq_out = tempfile.NamedTemporaryFile(suffix=".fastq").name
+        with open(fq_out, 'w') as fqh:
+            with open(fn, 'r') as fih:
+                for line in itertools.islice(fih, 240):
+                    fqh.write(line)
+        cset = ContigSet(fq_out)
+        cset_l = sum(1 for _ in cset)
+        self.assertEqual(cset_l, 60)
+        cset.filters.addRequirement(length=[('>', 1000)])
+        cset_l = sum(1 for _ in cset)
+        self.assertEqual(cset_l, 23)
+        cset.consolidate(cfq_out)
+        cset_l = sum(1 for _ in cset)
+        cfq = FastqReader(cfq_out)
+        self.assertEqual(cset_l, 23)
+        self.assertEqual(cset_l, sum(1 for _ in cfq))
+
 
     def test_len(self):
         # AlignmentSet
@@ -498,6 +616,8 @@ class TestDataSet(unittest.TestCase):
         aln.updateCounts()
         self.assertEqual(aln.totalLength, 123588)
         self.assertEqual(aln.numRecords, 92)
+        self.assertEqual(sum(1 for _ in aln), 92)
+        self.assertEqual(sum(len(rec) for rec in aln), 123588)
 
         # AlignmentSet with filters
         aln = AlignmentSet(data.getXml(15), strict=True)
@@ -514,18 +634,18 @@ class TestDataSet(unittest.TestCase):
         self.assertEqual(aln.numRecords, 40)
 
         # AlignmentSet with cmp.h5
-        aln = AlignmentSet(upstreamData.getCmpH5(), strict=True)
-        self.assertEqual(len(aln), 84)
-        self.assertEqual(aln._length, (84, 26103))
-        self.assertEqual(aln.totalLength, 26103)
-        self.assertEqual(aln.numRecords, 84)
+        aln = AlignmentSet(upstreamData.getBamAndCmpH5()[1], strict=True)
+        self.assertEqual(len(aln), 112)
+        self.assertEqual(aln._length, (112, 59970))
+        self.assertEqual(aln.totalLength, 59970)
+        self.assertEqual(aln.numRecords, 112)
         aln.totalLength = -1
         aln.numRecords = -1
         self.assertEqual(aln.totalLength, -1)
         self.assertEqual(aln.numRecords, -1)
         aln.updateCounts()
-        self.assertEqual(aln.totalLength, 26103)
-        self.assertEqual(aln.numRecords, 84)
+        self.assertEqual(aln.totalLength, 59970)
+        self.assertEqual(aln.numRecords, 112)
 
 
         # SubreadSet
@@ -541,6 +661,8 @@ class TestDataSet(unittest.TestCase):
         sset.updateCounts()
         self.assertEqual(sset.totalLength, 124093)
         self.assertEqual(sset.numRecords, 92)
+        self.assertEqual(sum(1 for _ in sset), 92)
+        self.assertEqual(sum(len(rec) for rec in sset), 124093)
 
         # HdfSubreadSet
         # len means something else in bax/bas land. These numbers may actually
@@ -602,7 +724,7 @@ class TestDataSet(unittest.TestCase):
 
     def test_nested_external_resources(self):
         log.debug("Testing nested externalResources in AlignmentSets")
-        aln = AlignmentSet(data.getXml(0))
+        aln = AlignmentSet(data.getXml(0), skipMissing=True)
         self.assertTrue(aln.externalResources[0].pbi)
         self.assertTrue(aln.externalResources[0].reference)
         self.assertEqual(
@@ -611,7 +733,7 @@ class TestDataSet(unittest.TestCase):
         self.assertEqual(aln.externalResources[0].scraps, None)
 
         log.debug("Testing nested externalResources in SubreadSets")
-        subs = SubreadSet(data.getXml(5))
+        subs = SubreadSet(data.getXml(5), skipMissing=True)
         self.assertTrue(subs.externalResources[0].scraps)
         self.assertEqual(
             subs.externalResources[0].externalResources[0].metaType,
@@ -626,6 +748,11 @@ class TestDataSet(unittest.TestCase):
         self.assertEqual(
             subs.externalResources[0].externalResources[0].metaType,
             'PacBio.SubreadFile.ScrapsBamFile')
+        subs.externalResources[0].barcodes = 'bc.fasta'
+        self.assertTrue(subs.externalResources[0].barcodes)
+        self.assertEqual(
+            subs.externalResources[0].externalResources[1].metaType,
+            "PacBio.DataSet.BarcodeSet")
 
         log.debug("Testing adding nested externalResources to AlignmetnSet "
                   "manually")
@@ -670,8 +797,100 @@ class TestDataSet(unittest.TestCase):
         self.assertEqual(ds[0].name, "lambda_NEB3011")
 
     def test_alignmentset_index(self):
-        aln = AlignmentSet(upstreamData.getCmpH5(), strict=True)
+        aln = AlignmentSet(upstreamData.getBamAndCmpH5()[1], strict=True)
         reads = aln.readsInRange(aln.refNames[0], 0, 1000)
         self.assertEqual(len(list(reads)), 2)
-        self.assertEqual(len(list(aln)), 84)
-        self.assertEqual(len(aln.index), 84)
+        self.assertEqual(len(list(aln)), 112)
+        self.assertEqual(len(aln.index), 112)
+
+    def test_barcodeset(self):
+        fa_out = tempfile.NamedTemporaryFile(suffix=".fasta").name
+        with open(fa_out, "w") as f:
+            f.write(">bc1\nAAAAAAAAAAAAAAAA\n>bc2\nCCCCCCCCCCCCCCCC")
+        ds = BarcodeSet(fa_out)
+        ds.induceIndices()
+        self.assertEqual([r.id for r in ds], ["bc1","bc2"])
+        ds_out = tempfile.NamedTemporaryFile(suffix=".barcodeset.xml").name
+        ds.write(ds_out)
+
+    @unittest.skipIf(not _internal_data(),
+                     "Internal data not found, skipping")
+    def test_barcode_split_cornercases(self):
+        fn = ('/pbi/dept/secondary/siv/testdata/'
+              'pblaa-unittest/Sequel/Phi29/m54008_160219_003234'
+              '.tiny.subreadset.xml')
+        sset = SubreadSet(fn)
+        ssets = sset.split(chunks=3, barcodes=True)
+        self.assertEqual([str(ss.filters) for ss in ssets],
+                         ["( bc = [0, 0] )",
+                          "( bc = [1, 1] )",
+                          "( bc = [2, 2] )"])
+        sset = SubreadSet(fn)
+        self.assertEqual(len(sset), 15133)
+        sset.filters = None
+        self.assertEqual(str(sset.filters), "")
+        sset.updateCounts()
+        self.assertEqual(len(sset), 2667562)
+
+        sset.filters.addRequirement(bc=[('=', '[2, 2]')])
+        self.assertEqual(str(sset.filters), "( bc = [2, 2] )")
+        sset.updateCounts()
+        self.assertEqual(len(sset), 4710)
+
+        sset.filters = None
+        self.assertEqual(str(sset.filters), "")
+        sset.updateCounts()
+        self.assertEqual(len(sset), 2667562)
+
+        sset.filters.addRequirement(bc=[('=', '[2,2]')])
+        self.assertEqual(str(sset.filters), "( bc = [2,2] )")
+        sset.updateCounts()
+        self.assertEqual(len(sset), 4710)
+
+    def test_merged_contigset(self):
+        fn = tempfile.NamedTemporaryFile(suffix=".contigset.xml").name
+        with ContigSet(upstreamData.getLambdaFasta(),
+                upstreamData.getFasta()) as cset:
+            self.assertEqual(len(list(cset)), 49)
+            self.assertEqual(len(cset), 49)
+            cset.consolidate()
+            cset.write(fn)
+            log.debug("Writing to {f}".format(f=fn))
+            self.assertEqual(len(list(cset)), 49)
+            self.assertEqual(len(cset), 49)
+        with ContigSet(fn) as cset:
+            self.assertEqual(len(list(cset)), 49)
+            self.assertEqual(len(cset), 49)
+
+    def test_incorrect_len_getitem(self):
+        types = [AlignmentSet(data.getXml(8)),
+                 ReferenceSet(data.getXml(9)),
+                 SubreadSet(data.getXml(10)),
+                 HdfSubreadSet(data.getXml(19))]
+        fn = tempfile.NamedTemporaryFile(suffix=".xml").name
+        for ds in types:
+            explen = -2
+            with openDataFile(ds.toExternalFiles()[0]) as mystery:
+                # try to avoid crashes...
+                explen = len(mystery)
+                mystery.numRecords = 1000000000
+                mystery.write(fn)
+            with openDataFile(fn) as mystery:
+                self.assertEqual(len(list(mystery)), explen)
+
+    def test_missing_fai_error_message(self):
+        outdir = tempfile.mkdtemp(suffix="dataset-unittest")
+
+        inFas = os.path.join(outdir, 'infile.fasta')
+
+        # copy fasta reference to hide fai and ensure FastaReader is used
+        backticks('cp {i} {o}'.format(
+            i=ReferenceSet(data.getXml(9)).toExternalFiles()[0],
+            o=inFas))
+        rs1 = ContigSet(inFas)
+        with self.assertRaises(IOError) as cm:
+            rs1.assertIndexed()
+        self.assertEqual(
+            str(cm.exception),
+            ( "Companion FASTA index (.fai) file not found or malformatted! "
+             "Use 'samtools faidx' to generate FASTA index."))
